import unittest
from input.get.journal_fetcher import JournalFetcher
from input.interface import InputInterface
from input.publication import Publication

"""
Testing the Publication fetcher
"""


txt_files=["acs.txt"]
supported_fetchers=["AcsFetcher","NatureFetcher"]


class InterfaceTestCase(unittest.TestCase):
    """
    Testcase for the InputInterface-class
    """

    def setUp(self):
        """
        Is called at the start of this TestCase
        """

        self.interface = InputInterface()


    def test_singleton(self):
        """
        Checks if InputInterface is a Singleton.
        """
        # Interface should already be made in setUp()
        self.assertNotEqual(self.interface.instance, None)
        # Checks if there is only one instance
        new_interface = InputInterface()
        self.assertEqual(self.interface, new_interface)
    

    def test_imported_modules(self):
        """
        Checks if every Fetcher has been automaticlly imported.
        """
        for fetcher in self.interface.get_supported_fetchers():
            self.assertIn(fetcher, supported_fetchers)


    def test_fetcher_selection(self):

        """
        fetches every txt-file in 'txt_files', extracts Doi-Url of the publication
        and then compares the the txt-string with the newly fetched one.
        """

        for txt in txt_files:
            f = open("input/test/test_txt/{}".format(txt),'r')
            txt_list = f.readlines()
            # Extracts the doi-url
            url = txt_list[1][8:].strip()
            # Creates new publication with the doi-url
            pub = self.interface.get_publication(url)
            self.maxDiff = None
            # Important: remove \n at the end, which is made by print()
            self.assertEqual("".join(txt_list),pub.__str__())
            f.close()   


class FetcherTestCase(unittest.TestCase):
    """
    Parent class for all journal specific fetchers.
    """

    def can_use_url_test(self, fetcher : JournalFetcher, test_url: str, expected_res: bool):
        """
        Tests the 'can_use_url'-method
        """ 
        self.assertEqual(fetcher.can_use_url(test_url), expected_res)


    def get_publication_test(self, fetcher : JournalFetcher, test_url: str, expected_res: Publication):
        """
        this test asserts that every variable is equals to the expected result
        """
        actual_res = fetcher.get_publication(test_url)
        self.assertEqual(actual_res.doi_url, expected_res.doi_url)
        self.assertEqual(actual_res.title, expected_res.title)
        self.assertEqual(actual_res.contributors, expected_res.contributors)
        self.assertEqual(actual_res.journal, expected_res.journal)
        self.assertEqual(actual_res.publication_date, expected_res.publication_date)
        self.assertEqual(actual_res.subjects, expected_res.subjects)
        self.assertEqual(actual_res.abstract, expected_res.abstract)

        # Checking for all references
        self.assertEqual(len(actual_res.references), len(expected_res.references))
        num_references = len(expected_res.references)
        for i in range(num_references):
            self.assertEqual(actual_res.references[i].doi_url,      expected_res.references[i].doi_url)
            self.assertEqual(actual_res.references[i].journal,      expected_res.references[i].journal)
            self.assertEqual(actual_res.references[i].contributors, expected_res.references[i].contributors)
            self.assertEqual(actual_res.references[i].cit_type,     expected_res.references[i].cit_type)

        # Checking for all citations
        self.assertEqual(len(actual_res.citations), len(expected_res.citations))
        num_citations = len(expected_res.citations)
        for i in range(num_citations):
            self.assertEqual(actual_res.citations[i].doi_url,      expected_res.citations[i].doi_url)
            self.assertEqual(actual_res.citations[i].journal,      expected_res.citations[i].journal)
            self.assertEqual(actual_res.citations[i].contributors, expected_res.citations[i].contributors)
            self.assertEqual(actual_res.citations[i].cit_type,     expected_res.citations[i].cit_type)


    def get_publication_exception_test(self, fetcher: JournalFetcher, test_url: str):
        """
        Checks if fetcher gives expected ValueError.
        """
        with self.assertRaises(ValueError):
            fetcher.get_publication(test_url)